@@ -33,6 +33,7 @@ class Engine:
         self.backend = kwargs.pop("backend", Backend.DEFAULT)
         self.raise_exceptions = kwargs.pop("raise_exceptions", True)
         self.deepcopy = kwargs.pop("deepcopy", True)
+        self.drop_substeps = kwargs.pop("drop_substeps", False)
 
         if kwargs:
             raise Exception(f"Invalid Engine option in {kwargs}")
@@ -169,23 +170,52 @@ class Engine:
                 simulation=simulation
             )
 
+            # NOTE Hook allows mutation of RunArgs
             for run_index in range(0, runs):
                 if param_sweep:
-                    for subset, param_set in enumerate(param_sweep):
-                        run_args = wrappers.Run(
+                    context = wrappers.Context(
+                        simulation_index,
+                        run_index,
+                        None,
+                        timesteps,
+                        initial_state,
+                        params
+                    )
+                    self.experiment._before_run(context=context)
+                    for subset_index, param_set in enumerate(param_sweep):
+                        context = wrappers.Context(
+                            simulation_index,
+                            run_index,
+                            subset_index,
+                            timesteps,
+                            initial_state,
+                            params
+                        )
+                        self.experiment._before_subset(context=context)
+                        yield wrappers.RunArgs(
                             simulation_index,
                             timesteps,
                             run_index,
-                            subset,
+                            subset_index,
                             copy.deepcopy(initial_state),
                             state_update_blocks,
                             copy.deepcopy(param_set),
-                            self.deepcopy
+                            self.deepcopy,
+                            self.drop_substeps,
                         )
-                        self.experiment._before_run(run=run_args)
-                        yield run_args
+                        self.experiment._after_subset(context=context)
+                    self.experiment._before_run(context=context)
                 else:
-                    run_args = wrappers.Run(
+                    context = wrappers.Context(
+                        simulation_index,
+                        run_index,
+                        0,
+                        timesteps,
+                        initial_state,
+                        params
+                    )
+                    self.experiment._before_run(context=context)
+                    yield wrappers.RunArgs(
                         simulation_index,
                         timesteps,
                         run_index,
@@ -193,12 +223,10 @@ class Engine:
                         copy.deepcopy(initial_state),
                         state_update_blocks,
                         copy.deepcopy(params),
-                        self.deepcopy
+                        self.deepcopy,
+                        self.drop_substeps,
                     )
-                    self.experiment._before_run(run=run_args)
-                    yield run_args
-
-                self.experiment._after_run(run=run_args)
+                    self.experiment._after_run(context=context)
 
             self.experiment._after_simulation(
                 simulation=simulation
